// Generated by LiveScript 1.5.0
var ref$, cyan, green, red, blue, inquirer, fs, glob, yaml, ServiceAdder, Logger, templatesPath, merge, N, path, flatten, reject, tmplconv, yamlCutter;
ref$ = require('chalk'), cyan = ref$.cyan, green = ref$.green, red = ref$.red, blue = ref$.blue;
inquirer = require('inquirer');
fs = require('fs');
glob = require('glob');
yaml = require('js-yaml');
ref$ = require('../../exosphere-shared'), ServiceAdder = ref$.ServiceAdder, Logger = ref$.Logger, templatesPath = ref$.templatesPath;
merge = require('merge');
N = require('nitroglycerin');
path = require('path');
ref$ = require('prelude-ls'), flatten = ref$.flatten, reject = ref$.reject;
tmplconv = require('tmplconv');
yamlCutter = require('yaml-cutter');
module.exports = function(){
  var ref$, data, questions, appConfig, e;
  if (process.argv[2] === "help") {
    return help();
  }
  console.log('We are about to add a new Exosphere service to the application!\n');
  ref$ = ServiceAdder.parseCommandLine(process.argv), data = ref$.data, questions = ref$.questions;
  try {
    appConfig = yaml.safeLoad(fs.readFileSync('application.yml', 'utf8'));
  } catch (e$) {
    e = e$;
    if (e.code === 'ENOENT') {
      console.log(red("Error: application.yml not found. Please run this command in the root directory of an Exosphere application."));
      process.exit();
    }
    throw e;
  }
  return inquirer.prompt(questions).then(function(answers){
    var srcPath, targetPath, pattern;
    data = merge(data, answers);
    checkForService({
      existingServices: getExistingServices(appConfig.services),
      serviceRole: data.serviceRole
    });
    srcPath = path.join(templatesPath, 'add-service', data.templateName);
    targetPath = path.join(process.cwd(), data.serviceType);
    data.appName = appConfig.name;
    pattern = ['**/*.*', '.dockerignore', 'Dockerfile'];
    return tmplconv.render(srcPath, targetPath, {
      data: data,
      pattern: pattern
    }).then(function(){
      var options;
      options = {
        file: 'application.yml',
        root: "services." + data.protectionLevel,
        key: data.serviceRole,
        value: {
          location: "./" + data.serviceType
        }
      };
      return yamlCutter.insertHash(options, N(function(){
        return console.log(green("\ndone"));
      }));
    });
  });
};
function help(){
  var helpMessage;
  helpMessage = "\nUsage: " + cyan('exo add') + " " + blue('[<entity-name>]') + "\n\nAdds a new service to the current application.\nThis command must be called in the root directory of the application.\n\noptions: " + blue('--service-role=[<service-role>] --service-type=[<service-type>] --template-name=[<template-name>] --model-name=[<model-name>] --protection-level=[<protection-level>] --description=[<description>]');
  return console.log(helpMessage);
}
function checkForService(arg$){
  var serviceRole, existingServices;
  serviceRole = arg$.serviceRole, existingServices = arg$.existingServices;
  if (existingServices.includes(serviceRole)) {
    console.log(red("Service " + cyan(serviceRole) + " already exists in this application"));
    return process.exit(1);
  }
}
function getExistingServices(services){
  var existingServices, protectionLevel, that;
  existingServices = [];
  for (protectionLevel in services) {
    if (that = services[protectionLevel]) {
      existingServices.push(Object.keys(that));
    }
  }
  return flatten(existingServices);
}